#pragma once
#include "FBullCowGame.h"
#include <map>
#define TMap std::map

using int32 = int;

FBullCowGame::FBullCowGame() {	Reset(); } //Constructor

int32 FBullCowGame::GetCurrentTry() const { return MyCurrentTries; }
int32 FBullCowGame::GetHiddenWordLength() const { return MyHiddenWord.length(); }
FString FBullCowGame::GetHiddenWord() {	return MyHiddenWord; }
bool FBullCowGame::IsGameWon() const { return BIsWon; }

void FBullCowGame::Reset() {	
	const FString HIDDEN_WORD = MyHiddenWord;// Set in SetHiddenWordbyDifficulty

	BIsWon = false;
	MyHiddenWord = HIDDEN_WORD;
	MyCurrentTries = 1;
	return;
}

int32 FBullCowGame::GetMaxTries() const { 
	TMap<int32, int32> WordLengthToMaxTries{ {3,4}, {4,6}, {5,10}, {6,12}, {15, 16} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

EWordStatus FBullCowGame::CheckGuessValidity(FString Guess) const {

	if (!IsIsogram(Guess)) {
		return EWordStatus::Not_Isogram;
	}
	else if (!IsLowercase(Guess)) {
		return EWordStatus::Not_Lowercase;
	}
	else if (Guess.length() != GetHiddenWordLength()) {
		return EWordStatus::Wrong_length;
	}
	else {
		return EWordStatus::OK;
	}
}

FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTries++;

	//initialise bullcowcount
	FBullCowCount BullCowCount;
	
	int32 HiddenWordLength = MyHiddenWord.length();
	//loop through all the letters of guess
	for (int32 i = 0; i < HiddenWordLength; i++) {
		//compare letters to the hidden word
		for (int32 j = 0; j < HiddenWordLength; j++) {	
			//if they match 
			if (MyHiddenWord[i] == Guess[j]) {
				//if they are in the same place 
				if (i == j) {
					BullCowCount.Bulls++;
				}
				else {					
					BullCowCount.Cows++;
				}
			}
		}		
	}
	if (BullCowCount.Bulls == GetHiddenWordLength()) {		
		BIsWon = true;
	}
	else {
		BIsWon = false;
	}
	return BullCowCount;
}

int32 FBullCowGame::SetHiddenWordbyDifficulty(int32 difficulty)
{	
	int randomnum4 = rand() % 4;
	const std::string threeletter[4] = { "and" ,"cab" ,"oat" ,"vas" };
	const std::string fourletter[4] = { "pork" ,"cake" ,"ogre" ,"take" };
	const std::string fiveletter[4] = { "madly" ,"habit" ,"padle" ,"ocean" };
	const std::string sixletter[4] = { "gamers" ,"darken" ,"kidnap" ,"quirky" };
	const std::string fifteenletter[2] = { "dermatoglyphics"};
	TMap<int32, std::string> DifficultyToHiddenWord{ 
		{1, threeletter[randomnum4]}, 
		{2, fourletter[randomnum4] },
		{3, fiveletter[randomnum4] },
		{4, sixletter[randomnum4] },
		{5, "dermatoglyphics" }
	};
	MyHiddenWord = DifficultyToHiddenWord[difficulty];
	return 0;
}

bool FBullCowGame::IsIsogram(FString Word) const {

	if (Word.length() <= 1) { return true; } // 1 and 0 words fall through

	TMap<char, bool> LetterSeen; //Declaring a Map

	for (auto Letter : Word) { //for every letter in Word
		Letter = tolower(Letter); //Converting every letter to lowercase

		if (LetterSeen[Letter]) { //check if letter seen
			return false; 
		}
		else { //otherwise add it to the map
			LetterSeen[Letter] = true;
		}
	}
	return true;
}

bool FBullCowGame::IsLowercase(FString Word) const {

	if (Word.length() <= 1) { return true; } // 1 and 0 words fall through

	for (auto letterF : Word) { //for every letter in Word

		if (!islower(letterF)) {	//Checking if letter lowercase			
			return false;
		}
	}	
	return true;
}

FString FBullCowGame::HiddenWordHint() const { // Gives Hint according to word
	TMap<std::string, std::string> WordHint{
		{"and", "Used to connect Words in sentences, Well you should know this :D" },
		{ "cab", "A taxi Or Simmilar to a taxi, BlacknYellow BlacknYellow :P" },
		{ "oat", "Cultivated in cool climates and widely used for animal feed, love these <3" },
		{ "vas", "A vessel or duct" },

		{"pork", "The flesh of a pig used as food, especially when uncured, EEasy :)" },
		{ "cake", "Used to Celebrate Birthdays" },
		{ "ogre", "a man-eating giant, HMMM :P" },
		{ "take", "lay hold of, IDK :3" },

		{"madly", "Adverb of mad, IDK :/" },
		{ "habit", "One thing that is hard to give up, Yeah i've got one.. :D" },
		{ "padle", "A garden hoe, When I saw this I thought it was inappropriate :D" },
		{ "ocean", "Largest area of water, Truth told ;)" },

		{"gamers", "Who plays :P" },
		{ "darken", "make or become dark or darker, It's in EN :O" },
		{ "kidnap", "abduct, Told you the answer its not abduct.. 3:)" },
		{ "quirky", "having or characterized by peculiar or unexpected traits or aspects, This is really hard 0:)" },

		{"dermatoglyphics", "the study of skin markings, HUH?" }
	};
	return WordHint[MyHiddenWord];
}
