#pragma once
#include <iostream>
#include <string>
#include "FBullCowGame.h"

using FText = std::string;
using int32 = int;

//Definition are after main()
void PrintIntro();
void PlayGame();
FText GetGuess();
bool AskToPlayAgain();
void PrintGameSummary();
int32 AskDifficulty();

FBullCowGame BCGame;//Initiating a game

//Entry point of our game
int main() {	
	bool bPlayAgain = false;
	do {	
		PrintIntro();
		PlayGame();
		bPlayAgain = AskToPlayAgain();
	} while (bPlayAgain);
	

	return 0;//exit point of application
}

//Definitions of Functions
void PrintIntro() {
	//Introducing the Game
	std::cout << "Welcome to Bulls and Cows, a fun word game.\n";
	std::cout << std::endl;
	std::cout << "          }   {         ___ " << std::endl;
	std::cout << "          (o o)        (o o) " << std::endl;
	std::cout << "   /-------\\ /          \\ /-------\\ " << std::endl;
	std::cout << "  / | BULL |O            O| COW  | \\ " << std::endl;
	std::cout << " *  |-,--- |              |------|  * " << std::endl;
	std::cout << "    ^      ^              ^      ^ " << std::endl;	
	AskDifficulty();
	std::cout << "Can you guess the " << BCGame.GetHiddenWordLength();
	std::cout << " letter isogram I'm thinking of?\n\n";
	std::cout << "Hint: " << BCGame.HiddenWordHint() << std::endl;
	std::cout << std::endl;
	return;
}

void PlayGame() {
	int32 MaxTries = BCGame.GetMaxTries();
	BCGame.Reset();
	//loops asking for Guess
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries) {
		FText Guess = GetGuess();
		EWordStatus Status = BCGame.CheckGuessValidity(Guess);		
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);

		// shows your score or bulls and cows
		std::cout << "Bulls : " << BullCowCount.Bulls;
		std::cout << ". Cows : " << BullCowCount.Cows << std::endl;
		std::cout << std::endl;
	}
	PrintGameSummary();
}

FText GetGuess() {
	int32 CurretTry = BCGame.GetCurrentTry();
	FText Guess = "";
	EWordStatus Status = EWordStatus::Invalid_Guess;
	do {
		//Take the Guess from the player
		std::cout << "Try " << CurretTry << " of " << BCGame.GetMaxTries() << ". Enter your Guess?? ";
		std::getline(std::cin, Guess);
		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status) {
		case EWordStatus::Wrong_length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter isogram" << std::endl;
			break;
		case EWordStatus::Not_Isogram:
			std::cout << "Please enter a isogram" << std::endl;
			break;
		case EWordStatus::Not_Lowercase:
			std::cout << "Please enter a all lowercase letters" << std::endl;
			break;
		default:			
			break;
		}
		std::cout << std::endl;
	} while (Status != EWordStatus::OK);	
	return Guess;
}

bool AskToPlayAgain() {
	FText Response = "";
	std::cout << "Do you want to play again? ";
	std::getline(std::cin, Response);
	return (Response[0] == 'y' || Response[0] == 'Y');
	
}

void PrintGameSummary() {
	int32 MAXTRIES = BCGame.GetMaxTries();
	if (BCGame.IsGameWon() == true) {
		std::cout << "You've won the game, CONGRATULATIONS! :)\n\n";		 
	}
	else if (BCGame.IsGameWon() == false) {
		std::cout << "Your word was: " << BCGame.GetHiddenWord() << std::endl;
		std::cout << "\nBetter Luck Next Time :P\n\n";
	}

}

int32 AskDifficulty() {
	std::string difficulty = "";
	std::cout << "Select Difficulty(1 to 5)? ";
	std::getline(std::cin, difficulty);
	if (difficulty.length() < 1) {
		return 1;
	}
	std::cout << std::endl;
	difficulty = BCGame.SetHiddenWordbyDifficulty(std::stoi(difficulty));
	return 0;
}